# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - ApproveChanges library

    This library relies on the existence of a user (by default
    "ApprovalQueueUser") who has sufficient privileges to write pages with ACLs
    to an approval queue (ACL permissions "write,admin").

    If users other than the superuser are to be able to edit pages freely, they
    must be present in a group (by default "ApprovedGroup"), and if they are to
    be allowed to review changes, they must be present in a different group (by
    default "PageReviewersGroup").

    @copyright: 2011 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin import user
from MoinMoin.wikiutil import escape
import re

acl_pattern = re.compile(ur"^#acl .*$", re.UNICODE | re.MULTILINE)

__version__ = "0.1"

def get_queued_changes_page(request):
    return getattr(request.cfg, "queued_changes_page", "ApprovalQueue")

def get_approved_editors_group(request):
    return getattr(request.cfg, "approved_editors_group", "ApprovedGroup")

def get_page_reviewers_group(request):
    return getattr(request.cfg, "reviewers_group", "PageReviewersGroup")

def get_queued_changes_user(request):
    return getattr(request.cfg, "queued_changes_user", "ApprovalQueueUser")

def get_secret_key(request):
    return request.cfg.secrets["wikiutil/tickets"]

def is_reviewer(request):
    return request.user.valid and (
        request.dicts.has_member(get_approved_editors_group(request), request.user.name) or \
        request.user.isSuperUser())

def is_approved(request):
    return request.user.valid and (
        request.dicts.has_member(get_approved_editors_group(request), request.user.name) or \
        request.user.isSuperUser())

def is_queued_changes_user(request):
    return request.user.valid and request.user.name == get_queued_changes_user(request)

def is_queued_changes_page(request, pagename):

    "Return whether 'pagename' is a queued changes page by testing its name."

    parts = pagename.split("/")
    return len(parts) > 1 and parts[-1] == get_queued_changes_page(request)

def get_target_page_name(pagename):

    "Return the target page name for the given queued changes 'pagename'."

    return "/".join(pagename.split("/")[:-1])

def get_user_for_saving(request):

    "Return a user that can save pages with ACLs."

    username = get_queued_changes_user(request)
    uid = user.getUserId(request, username)

    # If the user does not exist, just return the existing user.

    if not uid:
        return request.user

    # Otherwise, return the privileged user.

    return user.User(request, uid)

def add_access_control(request, body):

    """
    Using the 'request', add an ACL to the page 'body' in order to prevent
    anyone other than reviewers from seeing it in the queue.
    """

    # Find existing ACLs.

    match = acl_pattern.search(body)
    if match:
        start, end = match.span()

        # Comment out existing ACLs.

        parts = []
        parts.append(body[:start])
        parts.append("#")
        parts.append(body[start:])
    else:
        parts = [body]

    # Add the ACL.

    parts.insert(0, "#acl %s:read,write,delete,revert,admin All:\n" % 
        get_page_reviewers_group(request))
    return "".join(parts)

def remove_access_control(request, body):

    "Using the 'request', remove any added ACL to the page 'body'."

    lines = body.split("\n")

    try:
        directive = lines[0].split()[0]
        if directive == "#acl":
            return "\n".join(lines[1:])
    except ValueError:
        pass

    return body

# Utility classes and associated functions.
# NOTE: These are a subset of EventAggregatorSupport.

class Form:

    """
    A wrapper preserving MoinMoin 1.8.x (and earlier) behaviour in a 1.9.x
    environment.
    """

    def __init__(self, form):
        self.form = form

    def get(self, name, default=None):
        values = self.form.getlist(name)
        if not values:
            return default
        else:
            return values

    def __getitem__(self, name):
        return self.form.getlist(name)

class ActionSupport:

    """
    Work around disruptive MoinMoin changes in 1.9, and also provide useful
    convenience methods.
    """

    def get_form(self):
        return get_form(self.request)

def get_form(request):

    "Work around disruptive MoinMoin changes in 1.9."

    if hasattr(request, "values"):
        return Form(request.values)
    else:
        return request.form

def escattr(s):
    return escape(s, 1)

# vim: tabstop=4 expandtab shiftwidth=4
